package rocks.zipcode;

import java.io.*;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;
import java.util.Scanner;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author leon on 16/11/2018.
 */
public class Document implements DocumentInterface {

    private final FileWriter fileWriter;
    private String path;
    private final File file;

    public Document(String fileName) throws IOException {
        this(new File(fileName));
        this.path = fileName;
    }

    public Document(File file) throws IOException {
        this.file = file;
        this.fileWriter = new FileWriter(file);
    }

    @Override
    public void write(String contentToBeWritten) throws IllegalArgumentException {
        try {
            BufferedWriter bw = new BufferedWriter(new FileWriter(file));
            PrintWriter pw = new PrintWriter(bw);
            pw.write(contentToBeWritten);
            pw.flush();
        }catch(IOException e){
            System.out.println("unable to write to file.");
        }
    }

    @Override
    public void write(Integer lineNumber, String valueToBeWritten) {
        try {
            List<String> lines = Files.readAllLines(file.toPath());
            lines.set(lineNumber, valueToBeWritten);
            String content = "";
            for(int i = 0; i < lines.size(); i ++){
                if(i < lines.size() - 1) {
                    content += lines.get(i) + "\n";
                } else {
                    content += lines.get(i);
                }
            }
            overWrite(content);
        }catch(Exception e){
            System.out.println("An exception occurred");
        }
    }

    @Override
    public String read(Integer lineNumber) {
        BufferedReader reader = null;
        String result = null;
        try{
            reader = new BufferedReader(new FileReader(file));
            for(int i = 0; i <= lineNumber; i++){
                result = reader.readLine();
            }
            reader.close();
        }catch(FileNotFoundException e){
            System.out.println("Cannot find file.");
        }catch(IOException e){
            System.out.println("an I/O error occured.");
        }
        return result;
    }

    @Override
    public String read() {
        Scanner scanner = null;
        try {
            scanner = new Scanner(file);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        StringBuilder sb = new StringBuilder();
        if (scanner != null) {
            scanner.useDelimiter("\\\\n");
        }

        while (scanner.hasNext()) {
            sb.append(scanner.next());
        }

        return sb.toString();
    }

    @Override
    public void replaceAll(String stringToReplace, String replacementString) {
        Path path = Paths.get(file.getAbsolutePath());
        String fileContent;
        try {
            fileContent = new String(Files.readAllBytes(path));
            fileContent = fileContent.replaceAll(stringToReplace, replacementString);
            Files.write(path, fileContent.getBytes());
        } catch (IOException ioExcep) {
            ioExcep.printStackTrace();

        }
    }

    @Override
    public void overWrite(String content) {
        try {
            FileWriter fileWriter = new FileWriter(file, false);
            fileWriter.write(content);
            fileWriter.flush();
            fileWriter.close();
        } catch (IOException io){
            io.printStackTrace();
        }
    }

    public List<String> toList() {
        return null;
    }

    @Override
    public File getFile() {
        return null;
    }

    @Override
    public String toString() {
        return this.path + "{" + read() + "}";
    }

    public Boolean isSpecialChar(String s){
        Pattern pattern = Pattern.compile("(\\W+)");
        Matcher matcher = pattern.matcher(s);
        if(matcher.find()) {
            return true;
        }
        return false;
    }

    public Boolean isAlpha(String s){
        Pattern pattern = Pattern.compile("([a-zA-Z]+)");
        Matcher matcher = pattern.matcher(s);
        if(matcher.find()) {
            return true;
        }
        return false;
    }

    public Boolean isNumeric(String s) {
        Pattern pattern = Pattern.compile("\\d+");
        Matcher matcher = pattern.matcher(s);
        if (matcher.find()) {
            return true;
        }
        return false;
    }
}
